﻿//-----------------------------------------------------------------------------
// Copyright 2013 by VueSoft LLC. All rights reserved.
// http://www.printablecal.com
//-----------------------------------------------------------------------------
//
// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice, 
//    this list of conditions and the following disclaimer.
// 
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.
// 
// 3. Neither the name of "PrintableCal" nor the names of its contributors may 
//    be used to endorse or promote products derived from this software without
//    specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
// POSSIBILITY OF SUCH DAMAGE.
//
//-----------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Drawing;
using System.Globalization;
using System.Linq;
using System.Text;

using Excel = Microsoft.Office.Interop.Excel;

using Vuesoft.PrintableCal.Common;
using Vuesoft.PrintableCal.Common.Validation;
using Vuesoft.PrintableCal.Interfaces;

namespace Vuesoft.PrintableCal.Addins.Templates.AgendaTemplate
{
   /// <summary>
   /// This class provides an example template style. In this case, its the "Agenda" template style that is included with the 
   /// PrintableCal installation. The same techniques could be used to extend PrintableCal with entirely new template styles.
   /// 
   /// <remarks>
   /// When defining a template style, make sure to add the Export and ExportMetadata attributes. These are required for
   /// PrintableCal to recognize the type is an extension. 
   /// </remarks>
   /// </summary>
   [Export(typeof(ITemplate))]
   [ExportMetadata("Style", AgendaTemplate.STYLE)]
   public class AgendaTemplate : TemplateBase
   {
      /// <summary>
      /// The template style name. This name is used to group templates in the PrintableCal user interface. It also is used to
      /// determine which saved template file goes with which template style implementation. The style name must be unique!
      /// </summary>
      public const string STYLE = "Agenda";

      /// <summary>
      /// The identifier and label for the starting date template option.
      /// </summary>
      private static string OPTION_START_DATE = "Start date:";

      /// <summary>
      /// The identifier and label for the number of days option. This value is used as a key for persisting the option.
      /// </summary>
      private static string OPTION_NUMBER_OF_DAYS = "Number of days:";

      /// <summary>
      /// The identifier and label for the time format option. This value is used as a key for persisting the option.
      /// </summary>
      private static string OPTION_TIME_FORMAT = "Time format:";

      /// <summary>
      /// The identifier and label for the option to show descriptions. This value is used as a key for persisting the option.
      /// </summary>
      private static string OPTION_SHOW_DESCRIPTIONS = "Show descriptions";

      /// <summary>
      /// Get the icon to be displayed in the template list group header. Icons must be sized 24x24 pixels. For consistency, 
      /// they should show a simplified or notional representation of what the template will generate.
      /// </summary>
      public override Image Icon
      {
         get { return Properties.Resources.agenda; }
      }

      /// <summary>
      /// The default constructor.
      /// </summary>
      public AgendaTemplate()
         : base(STYLE)
      {
         DateTime date = DateTime.Today;

         // Add the template options. These will appear in the main PrintableCal user interface, after an Agenda template is 
         // selected and the Next button clicked. There are a number of different template option control types provided in
         // the Vuesoft.PrintableCal.Common namespace, depending on the type of option required.
         this.TemplateOptions.Add(new TemplateOptionDate(OPTION_START_DATE, date, false));
         this.TemplateOptions.Add(new TemplateOptionComboBox(OPTION_NUMBER_OF_DAYS, 7, 1, 365));
         this.TemplateOptions.Add(new TemplateOptionTimeFormat(OPTION_TIME_FORMAT, TimeFormat.Default, true));
         this.TemplateOptions.Add(new TemplateOptionCheckbox(OPTION_SHOW_DESCRIPTIONS, true));

         // Add the validation rules. These rules ensure that template files created by the user will be compatible with the 
         // Generate function defined in this class.
         this.ValidationRules.Add(new NamedCellValidation(
            "DayName", 
            "This name is used to determine where the day-of-week title should be placed in relation to other text for each generated day.",
            ValidationResultType.Error));

         this.ValidationRules.Add(new NamedCellValidation(
            "Date", 
            "This name is used to determine where the date should be placed in relation to other text for each generated day.",
            ValidationResultType.Error));

         this.ValidationRules.Add(new NamedCellValidation(
            "CalendarColor",
            "This name is used to determine where an item's color should be be placed in relation to the item text in each generated day.", 
            ValidationResultType.Warning));

         this.ValidationRules.Add(new NamedCellValidation(
            "Time", 
            "This name is used to determine where each item time should be placed in relation to the item title and description in each generated day.", 
            ValidationResultType.Error));

         this.ValidationRules.Add(new NamedCellValidation(
            "Title", 
            "This name is used to determine where each item title should be placed in relation to the item time and description in each generated day.", 
            ValidationResultType.Error));

         this.ValidationRules.Add(new NamedCellValidation(
            "Description",
            "This name is used to determine where each item description should be placed in relation to the item time and title in each generated day.",
            ValidationResultType.Error));

         this.ValidationRules.Add(new CellPositionValidation(
            "DayName",
            Direction.Above,
            new string[] { "CalendarColor", "Time", "Title", "Description" },
            "Items are listed under a header for each day. The day name is part of the header and must be above cells used for each item in the day.", 
            ValidationResultType.Error));

         this.ValidationRules.Add(new CellPositionValidation(
            "Date", 
            Direction.Above,
            new string[] { "CalendarColor", "Time", "Title", "Description" },
            "Items are listed under a header for each day. The date text is part of the header and must be above cells used for each item in the day.",
            ValidationResultType.Error));

         this.ValidationRules.Add(new WorksheetCountValidation(
            1, 
            "The entire Agenda template is defined on a single worksheet. Additional worksheets are unnecessary and may clutter the generated output.", 
            ValidationResultType.Warning));
         
         // The default Word output options can be modified. In this case, it makes more sense to show the agenda template in 
         // portrait orientation, rather than landscape.
         this.WordOptions.IsOrientationPortrait = true;
      }

      /// <summary>
      /// This function generates a calendar using the Agenda template style. The calendar is generated to the specified 
      /// workbook, using the specified calendar data sources.
      /// </summary>
      /// <param name="workbook">The output workbook.</param>
      /// <param name="calendars">The calendar data sources used to populate the output agenda.</param>
      /// <param name="status">The status object, updated with the current operation and percent completion as the template is
      /// being generated.</param>
      public override void Generate(Excel.Workbook workbook, ICalendarCollection calendars, IStatus status)
      {
         try
         {
            // get options
            DateTime startDate = (DateTime)GetTemplateOption(OPTION_START_DATE);
            int nDays = Convert.ToInt32(GetTemplateOption(OPTION_NUMBER_OF_DAYS));
            DateTime endDate = startDate.AddDays(nDays - 1);
            bool showDescriptions = Convert.ToBoolean(GetTemplateOption(OPTION_SHOW_DESCRIPTIONS));
            TimeFormat timeFormat = (TimeFormat)GetTemplateOption(OPTION_TIME_FORMAT);

            // initialize events
            status.StartOperation("Retrieving data from calendar sources...", 50);
            calendars.Initialize(startDate, endDate.AddDays(1).AddSeconds(-1), status);
            status.EndOperation();

            // figure out where values should be placed, using named range references
            status.StartOperation("Preparing workbook...", 60);
            Excel.Range templateCellDayName = workbook.Names.Item("DayName").RefersToRange;
            Excel.Range templateCellDate = workbook.Names.Item("Date").RefersToRange;
            Excel.Range templateCellCalendarColor = null;            
            Excel.Range templateCellTime = workbook.Names.Item("Time").RefersToRange;
            Excel.Range templateCellTitle = workbook.Names.Item("Title").RefersToRange;
            Excel.Range templateCellDescription = workbook.Names.Item("Description").RefersToRange;

            try
            {
               // the calendar color cell name is optional
               templateCellCalendarColor = workbook.Names.Item("CalendarColor").RefersToRange;
            }
            catch { }

            if (!showDescriptions)
            {
               templateCellDescription.Value = "";
               templateCellDescription = templateCellTitle;
            }

            DateTime currentDate = startDate;
            Excel.Worksheet ws = workbook.Sheets[1];

            int templateDateRowStart = MathUtils.MinValue(templateCellDayName.Row, templateCellDate.Row);
            int templateDateRowEnd = MathUtils.MaxValue(templateCellDayName.Row, templateCellDate.Row);
            int templateEventRowStart = 0;            
            int templateEventRowEnd = 0;

            if (templateCellCalendarColor != null)
            {
               templateEventRowStart = MathUtils.MinValue(templateCellCalendarColor.Row, templateCellTime.Row, templateCellTitle.Row, templateCellDescription.Row);
               templateEventRowEnd = MathUtils.MaxValue(templateCellCalendarColor.Row, templateCellTime.Row, templateCellTitle.Row, templateCellDescription.Row);
            }
            else
            {
               templateEventRowStart = MathUtils.MinValue(templateCellTime.Row, templateCellTitle.Row, templateCellDescription.Row);
               templateEventRowEnd = MathUtils.MaxValue(templateCellTime.Row, templateCellTitle.Row, templateCellDescription.Row);
            }

            int row = templateDateRowStart;
            status.EndOperation();

            double deltaPercent = 40.0 / ((endDate - startDate).TotalDays + 1);

            while (currentDate <= endDate)
            {
               status.StartOperation("Processing " + currentDate.ToLongDateString() + "...", Convert.ToInt32(status.PercentComplete + deltaPercent));
               List<IScheduleItem> items = calendars.GetItemsInRange(currentDate, currentDate.AddDays(1).AddSeconds(-1));

               if (items.Count > 0)
               {
                  // date row
                  if (row > templateDateRowStart)
                  {
                     Excel.Range copyRowStart = ws.Rows[templateDateRowStart];
                     Excel.Range copyRowEnd = ws.Rows[templateDateRowEnd];
                     ws.Range[copyRowStart.EntireRow, copyRowEnd.EntireRow].Copy(ws.Rows[row]);
                  }

                  // day name
                  Excel.Range dayNameCell = ws.Cells[row + templateCellDayName.Row - templateDateRowStart, templateCellDayName.Column];
                  dayNameCell.Value = currentDate;

                  // date
                  Excel.Range dateCell = ws.Cells[row + templateCellDate.Row - templateDateRowStart, templateCellDate.Column];
                  dateCell.Value = currentDate;

                  // padding
                  row += (templateDateRowEnd - templateDateRowStart) + 1;
                  Excel.Range paddingRow = ws.Rows[row];
                  paddingRow.EntireRow.RowHeight = 7.5;

                  // events
                  row += 1;

                  foreach (IScheduleItem item in items)
                  {
                     ScheduleItemCell calendarCell = new ScheduleItemCell();
                     calendarCell.Initialize(item, timeFormat, false, false, true, false);

                     if (row > templateEventRowStart)
                     {
                        Excel.Range copyRowStart = ws.Rows[templateEventRowStart];
                        Excel.Range copyRowEnd = ws.Rows[templateEventRowEnd];
                        ws.Range[copyRowStart.EntireRow, copyRowEnd.EntireRow].Copy(ws.Rows[row]);
                     }

                     // calendar color
                     if (templateCellCalendarColor != null)
                     {
                        Excel.Range calendarColorCell = ws.Cells[row + templateCellCalendarColor.Row - templateEventRowStart, templateCellCalendarColor.Column];
                        calendarColorCell.Interior.Color = ColorTranslator.ToOle(calendarCell.Appearance.BackgroundColor);

                        if (calendarCell.Appearance.BorderColor.A != 0)
                        {
                           int borderColor = ColorTranslator.ToOle(calendarCell.Appearance.BorderColor);
                           calendarColorCell.Borders.LineStyle = Excel.XlLineStyle.xlContinuous;
                           calendarColorCell.Borders.Color = borderColor;
                        }
                     }

                     // time
                     Excel.Range timeCell = ws.Cells[row + templateCellTime.Row - templateEventRowStart, templateCellTime.Column];

                     if (item.StartDateTime.Date != item.EndDateTime.Date)
                     {
                        if (item.IsAllDay)
                        {
                           timeCell.Value = "All day";
                        }
                        else
                        {
                           if (currentDate == item.StartDateTime.Date)
                           {
                              timeCell.Value = CalendarUtils.GetTimeString(timeFormat, item.StartDateTime) + " ►";
                           }
                           else if (currentDate == item.EndDateTime.Date)
                           {
                              timeCell.Value = "◄ " + CalendarUtils.GetTimeString(timeFormat, item.EndDateTime);
                           }
                           else
                           {
                              timeCell.Value = "◄ " + CalendarUtils.GetTimeString(timeFormat, currentDate) + " - " + 
                                                      CalendarUtils.GetTimeString(timeFormat, currentDate.AddMinutes(1339)) + "  ►";
                           }
                        }
                     }
                     else if (item.IsAllDay)
                     {
                        timeCell.Value = "All day";
                     }
                     else
                     {
                        timeCell.Value = CalendarUtils.GetTimeString(timeFormat,item.StartDateTime) + " - " + 
                                         CalendarUtils.GetTimeString(timeFormat,item.EndDateTime);
                     }

                     // title
                     Excel.Range titleCell = ws.Cells[row + templateCellTitle.Row - templateEventRowStart, templateCellTitle.Column];
                     titleCell.Value = calendarCell.Text;

                     // description
                     Excel.Range descriptionCell = ws.Cells[row + templateCellDescription.Row - templateEventRowStart, templateCellDescription.Column];

                     if (showDescriptions)
                     {
                        if (!String.IsNullOrEmpty(item.Description))
                        {
                           descriptionCell.Value = item.Description;
                           descriptionCell.EntireRow.AutoFit();
                           row += 2;

                           // padding
                           if (templateEventRowStart != templateEventRowEnd)
                           {
                              paddingRow = ws.Rows[row];
                              paddingRow.EntireRow.RowHeight = 7.5;
                           }

                           row -= 1;
                        }
                        else
                        {
                           descriptionCell.Value = "";

                           if (templateEventRowStart != templateEventRowEnd)
                           {
                              descriptionCell.EntireRow.RowHeight = 7.5;
                           }
                        }
                     }
                     else
                     {
                        descriptionCell.Value = "";
                     }

                     row += (templateEventRowEnd - templateEventRowStart + 1);
                  }

                  row += 1;

                  // padding
                  paddingRow = ws.Rows[row];
                  paddingRow.EntireRow.RowHeight = 7.5;
               }

               currentDate = currentDate.AddDays(1);
               status.EndOperation();
            }
         }
         catch (Exception ex)
         {
            FormError.ShowError(ex);
         }
      }
   }
}
